//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/View/Realspace/RealSpaceWidget.cpp
//! @brief     Implements class RealSpaceWidget
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/View/Realspace/RealSpaceWidget.h"
#include "GUI/View/Common/StyledToolBar.h"
#include "GUI/View/Realspace/RealSpaceCanvas.h"
#include <QLabel>
#include <QVBoxLayout>

RealSpaceWidget::RealSpaceWidget(SampleModel* sampleModel, QItemSelectionModel* selectionModel,
                                 QWidget* parent)
    : QWidget(parent)
    , m_canvas(new RealSpaceCanvas)
    , m_sampleModel(sampleModel)
    , m_selectionModel(selectionModel)
{
    createActions();

    auto hlayout = new QHBoxLayout;
    hlayout->setMargin(0);
    hlayout->setSpacing(0);
    hlayout->setContentsMargins(0, 0, 0, 0);
    hlayout->addWidget(m_canvas);

    auto* toolbar = new StyledToolBar(this);
    toolbar->addAction(m_savePictureAction);
    toolbar->addSeparator();
    toolbar->addAction(m_defaultViewAction);
    toolbar->addSeparator();
    toolbar->addAction(m_sideViewAction);
    toolbar->addSeparator();
    toolbar->addAction(m_topViewAction);
    toolbar->addSeparator();
    toolbar->addAction(m_increaseLayerSizeAction);
    toolbar->addSeparator();
    toolbar->addAction(m_decreaseLayerSizeAction);
    toolbar->addSeparator();
    toolbar->addAction(m_lockViewAction);

    auto* mainLayout = new QVBoxLayout;
    mainLayout->setMargin(0);
    mainLayout->setSpacing(0);
    mainLayout->setContentsMargins(0, 0, 0, 0);
    mainLayout->addWidget(toolbar);
    mainLayout->addLayout(hlayout);

    setLayout(mainLayout);

    connect(m_defaultViewAction, &QAction::triggered, m_canvas,
            &RealSpaceCanvas::onDefaultViewAction);

    connect(m_sideViewAction, &QAction::triggered, m_canvas, &RealSpaceCanvas::onSideViewAction);

    connect(m_topViewAction, &QAction::triggered, m_canvas, &RealSpaceCanvas::onTopViewAction);

    connect(m_lockViewAction, &QAction::triggered, m_canvas, &RealSpaceCanvas::onLockViewAction);

    connect(m_increaseLayerSizeAction, &QAction::triggered,
            [this]() { m_canvas->onChangeLayerSizeAction(1.25); });

    connect(m_decreaseLayerSizeAction, &QAction::triggered,
            [this]() { m_canvas->onChangeLayerSizeAction(0.8); });

    connect(m_savePictureAction, &QAction::triggered, m_canvas,
            &RealSpaceCanvas::onSavePictureAction);
}

void RealSpaceWidget::showEvent(QShowEvent*)
{
    m_canvas->setModel(m_sampleModel, m_selectionModel);
}

void RealSpaceWidget::hideEvent(QHideEvent*)
{
    m_canvas->setModel(nullptr, nullptr);
}

void RealSpaceWidget::createActions()
{
    m_savePictureAction = new QAction(this);
    m_savePictureAction->setText("Save Picture");
    m_savePictureAction->setToolTip("Save 3D real space view as .png file");

    m_defaultViewAction = new QAction(this);
    m_defaultViewAction->setText("Default View");
    m_defaultViewAction->setToolTip("Reset view and zoom level to default");

    m_sideViewAction = new QAction(this);
    m_sideViewAction->setText("Side View");
    m_sideViewAction->setToolTip("View sample from the side at current zoom level");

    m_topViewAction = new QAction(this);
    m_topViewAction->setText("Top View");
    m_topViewAction->setToolTip("View sample from the top at current zoom level");

    m_lockViewAction = new QAction(this);
    m_lockViewAction->setText("Lock View");
    m_lockViewAction->setToolTip("Lock/unlock current sample selection");
    m_lockViewAction->setCheckable(true);

    m_increaseLayerSizeAction = new QAction(this);
    m_increaseLayerSizeAction->setText("Enlarge");
    m_increaseLayerSizeAction->setToolTip("Increase layer size");

    m_decreaseLayerSizeAction = new QAction(this);
    m_decreaseLayerSizeAction->setText("Reduce");
    m_decreaseLayerSizeAction->setToolTip("Decrease layer size");
}
